""""

Test examples

"""


def test_register_asset():
    from examples.register_asset import main
    main()

def test_register_memory_asset():
    from examples.register_memory_asset import main
    main()


def test_surfer_memory_asset():
    from examples.surfer_memory_asset import main
    main()
